"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.selectDeploymentGroups = void 0;
const config_sets_1 = require("@takomo/config-sets");
const util_1 = require("@takomo/util");
const ramda_1 = __importDefault(require("ramda"));
const create_deployment_target_name_pattern_matcher_1 = require("./create-deployment-target-name-pattern-matcher");
const sortGroups = (a, b) => {
    const order = a.priority - b.priority;
    return order !== 0 ? order : a.name.localeCompare(b.name);
};
const selectDeploymentGroups = (ctx, props) => {
    const { configSetName, commandPath, configSetType, groups, targets, excludeTargets, labels, excludeLabels, } = props;
    if (groups.length > 0) {
        groups.forEach((groupPath) => {
            if (!ctx.hasDeploymentGroup(groupPath)) {
                throw new util_1.TakomoError(`Deployment group '${groupPath}' not found`);
            }
        });
    }
    if (commandPath && !configSetName) {
        throw new util_1.TakomoError("If you provide command path, you must provide config set, too");
    }
    if (configSetName) {
        if (!ctx.hasConfigSet(configSetName)) {
            throw new util_1.TakomoError(`Config set '${configSetName}' not found`);
        }
    }
    const rootGroups = groups.length === 0
        ? ctx.rootDeploymentGroups
        : groups.reduce((collected, path) => [...collected, ctx.getDeploymentGroup(path)], new Array());
    const flattenedGroups = rootGroups
        .map((group) => (0, util_1.collectFromHierarchy)(group, (o) => o.children, {
        sortSiblings: sortGroups,
        filter: (o) => o.status === "active",
    }))
        .flat();
    const uniqueGroups = ramda_1.default.uniqBy(ramda_1.default.prop("path"), flattenedGroups).filter((group) => group.status === "active");
    const configSetsMatch = (a) => {
        if (!configSetType) {
            return true;
        }
        const configSets = (0, config_sets_1.getConfigSetsByType)(configSetType, a);
        if (configSets.length === 0) {
            return false;
        }
        if (configSetName) {
            if (!configSets.some(({ name }) => name === configSetName)) {
                return false;
            }
        }
        return true;
    };
    const isActive = ({ status }) => status === "active";
    const targetNameMatchers = targets.map(create_deployment_target_name_pattern_matcher_1.createDeploymentTargetNamePatternMatcher);
    const excludeTargetNameMatchers = excludeTargets.map(create_deployment_target_name_pattern_matcher_1.createDeploymentTargetNamePatternMatcher);
    const targetNameMatches = (target) => (targetNameMatchers.length === 0 ||
        targetNameMatchers.some((m) => m(target))) &&
        (excludeTargetNameMatchers.length === 0 ||
            !excludeTargetNameMatchers.some((m) => m(target)));
    const labelMatches = (target) => (labels.length === 0 || target.labels.some((l) => labels.includes(l))) &&
        (excludeLabels.length === 0 ||
            !target.labels.some((l) => excludeLabels.includes(l)));
    const filterTargetsBySelectionCriteria = (targets) => targets.filter((target) => isActive(target) &&
        configSetsMatch(target) &&
        targetNameMatches(target) &&
        labelMatches(target));
    return uniqueGroups
        .map((group) => ({
        ...group,
        targets: filterTargetsBySelectionCriteria(group.targets),
    }))
        .filter(({ targets }) => targets.length > 0);
};
exports.selectDeploymentGroups = selectDeploymentGroups;
//# sourceMappingURL=select-deployment-groups.js.map